//
// Copyright 2007, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.


#include <xlate.h>
#include <list>
#include <utility>
#include <iostream>

using namespace std;

extern char const *programName;

typedef list<pair<int,int> > RangeList;

TranslateSet::TranslateSet()
{
  for(int i=0; i < 255; ++i)
    inset_[i] = char(i);
}


static bool populateCharSets(string const&  set1, 
                             string const&  set2, 
                             unsigned char* inset,
                             unsigned char* outset
                            );



TranslateSet::TranslateSet(string const &inset, string const &outset)
{
  unsigned char outset_[256];

  for(int i=0; i < 255; ++i)
    inset_[i] = outset_[i] = char(i);

  if( populateCharSets(inset, outset, inset_, outset_) )
  {
    cerr << programName << "  Error;:  input and output character sets are incompatible\n"
                           "           or badly formatted:\n"
                           "             '" << inset << "'\n"
                        << "             '" << outset<< "'\n"
                        << endl;

    exit(1);
  }

  // inset is now modified so that it is the mapping between plain text characters
  // and the final output form.

}

string 
TranslateSet::
translate(string const &s) const
{
  string rv;

  string::const_iterator a, b;

  for(a = s.begin(), b = s.end(); a != b; ++a)
  {
    char c = *a;

    unsigned char u = inset_[ static_cast<unsigned char>(c) ];

    c = static_cast<char>(u);

    rv += c;

  }


  return rv;
}


static void populateCharRange(string const &description, RangeList &l);
  //.  Parse the string, description, into a RangeList as a list of pairs of integer
  //.  ranges (incluse - exclusive).

static bool populateCharSets(string const&  set1, 
                             string const&  set2, 
                             unsigned char* inset,
                             unsigned char* outset
                            )
{
  //.  Populate two character sets with corresponding character range translations
  //.  or produce an error message and return failure.

  // this code assumes that inset and outset are already populated with simple ascending
  // orders of numbers in the range 0-255.

  RangeList inrange;
  RangeList outrange;


  populateCharRange(set1, inrange);
  populateCharRange(set2, outrange);

  if(inrange.size() != outrange.size())
  {
    return true;
  }

  RangeList::const_iterator a, b, l, g;

  for(a = inrange.begin(), b = inrange.end(), l = outrange.begin(), g = outrange.end();
      a != b; 
      ++a, ++l
     )
  {
    // since the lists have the same size, the iterators will walk along in lock step

    int inWidth  = a->second - a->first;   // width's are 1 less than true width
    int outWidth = l->second - l->first;

    if(inWidth != outWidth)
    {
      return true;

    }

    // ok, at this point we know that range a(first, second) == l(first,second)


    ++inWidth;

    // the width's are now true widths

    // table outset is ordered 0 - 255, and at this point the inset is also mapped this way,
    // but we are going to change inset so that it translates slots in inset to different
    // slots int the outset.
    //

    int inIndex  = a->first;
    int outIndex = l->first;

    while(inWidth--)
    {
      inset[inIndex++] = outIndex++;
    }

  }

  return false;

}
static void populateCharRange(string const &description, RangeList &l)
{
  string::const_iterator a = description.begin(), b = description.end();

  if(a != b)
  {
    if(*a == '-')
    {
      l.push_back( pair<int,int>('-', '-') );
      ++a;
    }
  }

  while(a != b)
  {
    char first = *a++;
    char second= first;

    if(a != b && *a == '-')
    {
      // we have a range
    
      ++a;
    
      if(a != b)
      {
        second = *a++;
      }
    
    }
    
    l.push_back( pair<int,int>(first, second) );

  }
}




